C vc3to2.f is a part of the VINCIA plug-in to the PYTHIA 8 generator.
C VINCIA is licenced under the GNU GPL version 2.
C Copyright (C) 2015 P. Skands
C Please respect the MCnet Guidelines for Event Generator Authors and Users, 
C               http://www.montecarlonet.org/GUIDELINES
      SUBROUTINE VC3TO2(MMAP,PA,PR,PB,PAHAT,PBHAT,IFAIL)
C...FORTRAN SUBROUTINE TO CLUSTER 3->2 A LA VINCIA
C...Last change: 17/01 2010, P. Skands
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
C...Input:C...  mmap: select which kinematics map to use
C...  pa, pr, pb : momenta to be clustered (px,py,pz,E)
      DOUBLE PRECISION PA(4),PR(4),PB(4)
      INTEGER MMAP
C...Output:
C...  pahat, pbhat : original momenta (px,py,pz,E)
C...  ifail : status return code, 0 = ok, > 0 = warning, < 0 = error.
      DOUBLE PRECISION PAHAT(4),PBHAT(4)      
C...Local variables:
      DOUBLE PRECISION PADUM(4), PRDUM(4), PBDUM(4), PSUM(4),BOOST(3)
      PARAMETER (PI=3.141592653589793D0)

C...Start by assuming everything will be fine
      IFAIL=0

C...Make copies of PA, PR, PB 
C...and compute sum of LAB momenta and CM energy
      DO 10 J=1,4
        PADUM(J) = PA(J)
        PRDUM(J) = PR(J)
        PBDUM(J) = PB(J)
        PSUM(J)  = PA(J)+PR(J)+PB(J)
 10   CONTINUE
      S = PSUM(4)**2-PSUM(1)**2-PSUM(2)**2-PSUM(3)**2
      IF (S.LE.1D-20) THEN 
C        WRITE(*,*) '(VC3TO2:) Sorry, massless or ' 
C     &    //' spacelike system. Cannot find rest frame' 
        IFAIL=-1
        RETURN
      ENDIF
      ECM=SQRT(S)

C...Compute boost from CM to LAB
      DO 20 J=1,3
        BOOST(J)=PSUM(J)/PSUM(4)
 20   CONTINUE

C...Boost local copies back to CM
C      print*, '   LAB momenta: '
C      print*, '     a   ',PADUM
C      print*, '     r   ',PRDUM
C      print*, '     b   ',PBDUM
      CALL VCROBO(PADUM,0D0,0D0,-BOOST(1),-BOOST(2),-BOOST(3))
      CALL VCROBO(PRDUM,0D0,0D0,-BOOST(1),-BOOST(2),-BOOST(3))
      CALL VCROBO(PBDUM,0D0,0D0,-BOOST(1),-BOOST(2),-BOOST(3))
C      print*, ' ECM = ',ECM
C      print*, '   CM momenta:'
C      print*, '     a   ',PADUM
C      print*, '     r   ',PRDUM
C      print*, '     b   ',PBDUM
C      print*, '   sum = ',(PADUM(J)+PRDUM(J)+PBDUM(J),J=1,4)

C...Rotate so a goes into upper half of (x,z) plane
C...(also rotate b, since rotated momentum needed below)
      PHIA=VCANGL(PADUM(1),PADUM(2))
      CALL VCROBO(PADUM,0D0,-PHIA,0D0,0D0,0D0)
      CALL VCROBO(PBDUM,0D0,-PHIA,0D0,0D0,0D0)
C      CALL VCROBO(PRDUM,0D0,-PHIA,0D0,0D0,0D0) !
C      print*, '   CM (phia) momenta:'
C      print*, '     a   ',PADUM
C      print*, '     r   ',PRDUM
C      print*, '     b   ',PBDUM
C...Rotate so a goes onto z axis
C...(only rotate b now, since only rot-b momentum needed below)
      THETA=VCANGL(PADUM(3),PADUM(1))
C      CALL VCROBO(PADUM,-THETA,0D0,0D0,0D0,0D0) !
      CALL VCROBO(PBDUM,-THETA,0D0,0D0,0D0,0D0)
C      CALL VCROBO(PRDUM,-THETA,0D0,0D0,0D0,0D0) !
C...Rotate so (r,b) goes into (x,z) plane
      PHIB=VCANGL(PBDUM(1),PBDUM(2))
C      CALL VCROBO(PADUM,0D0,-PHIB,0D0,0D0,0D0) !
C      CALL VCROBO(PRDUM,0D0,-PHIB,0D0,0D0,0D0) ! 
C      CALL VCROBO(PBDUM,0D0,-PHIB,0D0,0D0,0D0) !
C      print*, '  phia  =',phia
C      print*, '  theta =',theta
C      print*, '  phib  =',phib
C      print*, '   CM(x,z) momenta:'
C      print*, '     a   ',PADUM
C      print*, '     r   ',PRDUM
C      print*, '     b   ',PBDUM

C...Compute psi(a,ahat) from A, B energies and theta(AB) 
      THEAB=VCANGL(PBDUM(3),SQRT(PBDUM(1)**2+PBDUM(2)**2))
      PSI=0D0
C...ARIADNE Angle (smoothly varying antenna recoil)
      IF (MMAP.EQ.1) THEN
        PSI=PBDUM(4)*PBDUM(4)/(PADUM(4)*PADUM(4)+PBDUM(4)*PBDUM(4))
     &      *(PI-THEAB)
C...PS Angle (a direction fixed if sar > srb, and vice versa)
C...(use org momenta, since new ones not all rotated)
      ELSE IF (MMAP.EQ.2) THEN
        SAR = (PA(4)+PR(4))**2-(PA(1)+PR(1))**2
     &      -(PA(2)+PR(2))**2-(PA(3)+PR(3))**2
        SRB = (PB(4)+PR(4))**2-(PB(1)+PR(1))**2
     &       -(PB(2)+PR(2))**2-(PB(3)+PR(3))**2
C        print*, '  MAR =',sqrt(sar), ' MRB =',sqrt(srb)
        IF (SAR.GT.SRB) THEN
C...B is the emitter. A recoils longitudinally.
          PSI=0D0
        ELSE 
C...A is the emitter. B recoils longitudinally.
          PSI=(PI-THEAB)
        ENDIF
C        print*, ' PSI = ',PSI
      ELSEIF (MMAP.EQ.-1) THEN
C...Force A to be the emitter. B recoils longitudinally.
        PSI=(PI-THEAB)
      ELSEIF (MMAP.EQ.-2) THEN
C...Force B to be the emitter. A recoils longitudinally.
        PSI=0D0
      ENDIF
C      print*, '  theAB =',theab, '     ea^2 =',PADUM(4)*PADUM(4),
C     &    '   eb^2 =',PBDUM(4)*PBDUM(4) 
C      print*, '  psi =',psi

C...Now we know everything:
C...  CM -> LAB : -PSI, PHIB, THETA, PHIA, BOOST

C...Set up initial massless AHAT, BHAT with PAHAT along z axis
      PAHAT(1) = 0D0
      PBHAT(1) = 0D0
      PAHAT(2) = 0D0
      PBHAT(2) = 0D0
      PAHAT(3) = ECM/2D0
      PBHAT(3) =-ECM/2D0
      PAHAT(4) = ECM/2D0
      PBHAT(4) = ECM/2D0

C...Take into account antenna recoil
      CALL VCROBO(PAHAT,-PSI,0D0,0D0,0D0,0D0)
      CALL VCROBO(PBHAT,-PSI,0D0,0D0,0D0,0D0)
C...1) Rotate back in PHIB
      CALL VCROBO(PAHAT,0D0,PHIB,0D0,0D0,0D0)
      CALL VCROBO(PBHAT,0D0,PHIB,0D0,0D0,0D0)
C...2) Rotate back in THETA
      CALL VCROBO(PAHAT,THETA,0D0,0D0,0D0,0D0)
      CALL VCROBO(PBHAT,THETA,0D0,0D0,0D0,0D0)
C...3) Rotate back in PHIA
      CALL VCROBO(PAHAT,0D0,PHIA,0D0,0D0,0D0)
      CALL VCROBO(PBHAT,0D0,PHIA,0D0,0D0,0D0)

C      print*, ' CM(rot) hat momenta:'
C      print*, '     a-hat ',PAHAT
C      print*, '     b-hat ',PBHAT

C...Boost back to LAB
      CALL VCROBO(PAHAT,0D0,0D0,BOOST(1),BOOST(2),BOOST(3))
      CALL VCROBO(PBHAT,0D0,0D0,BOOST(1),BOOST(2),BOOST(3))
 
C      print*, ' LAB hat momenta:'
C      print*, '     a-hat ',PAHAT
C      print*, '     b-hat ',PBHAT
c      print*, '   sum = ',(PAHAT(J)+PBHAT(J),J=1,4)

      RETURN
      END

C*********************************************************************
 
C...VCROBO
C...Adapted from PYROBO by T. Sjostrand
C...Performs rotations and boosts.
 
      SUBROUTINE VCROBO(P,THE,PHI,BEX,BEY,BEZ) 
C...Double precision and integer declarations.
      IMPLICIT DOUBLE PRECISION(A-H, O-Z)
      IMPLICIT INTEGER(I-N)
      DIMENSION P(4)
C...Local arrays.
      DIMENSION ROT(3,3),PR(3),DP(4)
 
C...Rotate, typically from z axis to direction (theta,phi).
      IF(THE**2+PHI**2.GT.1D-20) THEN
        ROT(1,1)=COS(THE)*COS(PHI)
        ROT(1,2)=-SIN(PHI)
        ROT(1,3)=SIN(THE)*COS(PHI)
        ROT(2,1)=COS(THE)*SIN(PHI)
        ROT(2,2)=COS(PHI)
        ROT(2,3)=SIN(THE)*SIN(PHI)
        ROT(3,1)=-SIN(THE)
        ROT(3,2)=0D0
        ROT(3,3)=COS(THE)
        DO 120 J=1,3
          PR(J)=P(J)
 120    CONTINUE
        DO 130 J=1,3
          P(J)=ROT(J,1)*PR(1)+ROT(J,2)*PR(2)+ROT(J,3)*PR(3)
 130    CONTINUE
      ENDIF
        
C...Boost, typically from rest to momentum/energy=beta.
      IF(BEX**2+BEY**2+BEZ**2.GT.1D-20) THEN
        DBX=BEX
        DBY=BEY
        DBZ=BEZ
        DB=SQRT(DBX**2+DBY**2+DBZ**2)
        EPS1=1D0-1D-12
        IF(DB.GT.EPS1) THEN
C...Rescale boost vector if too close to unity.
C          WRITE(*,*) '(VCROBO:) boost vector too large'
          DBX=DBX*(EPS1/DB)
          DBY=DBY*(EPS1/DB)
          DBZ=DBZ*(EPS1/DB)
          DB=EPS1
        ENDIF
        DGA=1D0/SQRT(1D0-DB**2)
        DO 150 J=1,4
          DP(J)=P(J)
 150    CONTINUE
        DBP=DBX*DP(1)+DBY*DP(2)+DBZ*DP(3)
        DGABP=DGA*(DGA*DBP/(1D0+DGA)+DP(4))
        P(1)=DP(1)+DGABP*DBX
        P(2)=DP(2)+DGABP*DBY
        P(3)=DP(3)+DGABP*DBZ
        P(4)=DGA*(DP(4)+DBP)
      ENDIF
 
      RETURN
      END

 
C*********************************************************************
 
C...VCANGL
C...Adapted from PYANGL by T. Sjostrand
C...Reconstructs an angle from given x and y coordinates.
 
      FUNCTION VCANGL(X,Y)
 
C...Double precision and integer declarations.
      IMPLICIT DOUBLE PRECISION(A-H, O-Z)
      IMPLICIT INTEGER(I-N)
      PARAMETER (PI=3.141592653589793D0)

      VCANGL=0D0
      R=SQRT(X**2+Y**2)
      IF(R.LT.1D-20) RETURN
      IF(ABS(X)/R.LT.0.8D0) THEN
        VCANGL=SIGN(ACOS(X/R),Y)
      ELSE
        VCANGL=ASIN(Y/R)
        IF(X.LT.0D0.AND.VCANGL.GE.0D0) THEN
          VCANGL=PI-VCANGL
        ELSEIF(X.LT.0D0) THEN
          VCANGL=-PI-VCANGL
        ENDIF
      ENDIF
 
      RETURN
      END
